@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: app.h,v 1.15 2001/06/02 01:26:38 blais Exp $
- * $Date: 2001/06/02 01:26:38 $
+ * $Id: app.h,v 1.52 2002/02/11 02:25:21 blais Exp $
+ * $Date: 2002/02/11 02:25:21 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *****************************************************************************/
+ ******************************************************************************/
 
 #ifndef INCL_XXDIFF_APP
 #define INCL_XXDIFF_APP
@@ -39,6 +39,10 @@
 #include <resources.h>
 #endif
 
+#ifndef INCL_XXDIFF_RESPARSER
+#include <resParser.h>
+#endif
+
 #ifndef INCL_QT_QAPPLICATION
 #include <qapplication.h>
 #define INCL_QT_QAPPLICATION
@@ -49,11 +53,6 @@
 #define INCL_STD_MEMORY
 #endif
 
-#ifndef INCL_STD_STRING
-#include <string>
-#define INCL_STD_STRING
-#endif
-
 #ifndef INCL_STD_LIST
 #include <list>
 #define INCL_STD_LIST
@@ -64,12 +63,12 @@
  *============================================================================*/
 
 class QMainWindow;
-class QScrollBar;
-class QLabel;
-class QDialog;
 class QSocketNotifier;
 class QToolBar;
 class QMessageBox;
+class QSplitter;
+class QLabel;
+class QPopupMenu;
 
 XX_NAMESPACE_BEGIN
 
@@ -77,16 +76,18 @@ XX_NAMESPACE_BEGIN
  * FORWARD DECLARATIONS
  *============================================================================*/
 
+class XxCmdline;
 class XxOverview;
 class XxLineNumbers;
 class XxText;
+class XxMergedFrame;
 class XxMergedWindow;
 class XxDiffs;
 class XxBuffer;
 class XxResources;
 class XxOptionsDialog;
-class XxStringResParser;
 class XxSearchDialog;
+class XxCentralFrame;
 
 /*==============================================================================
  * CLASS XxApp
@@ -107,7 +108,7 @@ public:
    /*----- member functions -----*/
 
    // Constructor.
-   XxApp( int argc, char** argv, bool forceStyle = false );
+   XxApp( int& argc, char** argv, const XxCmdline& );
 
    // Destructor.
    virtual ~XxApp();
@@ -119,64 +120,37 @@ public:
    uint getNbFiles() const;
 
    // Returns the file for requested buffer.
-   // FIXME shouldn't you change this for getBuffer()?  do it everywhere too.
-   XxBuffer* getFile( const XxFno no ) const;
+   XxBuffer* getBuffer( const XxFno no ) const;
 
    // Returns an array of the file pointers.
-   const std::auto_ptr<XxBuffer>* getFiles() const;
+   const std::auto_ptr<XxBuffer>* getBuffers() const;
 
    // Returns the maximum number of digits for line numbers of all files.
    uint getMaxDigits() const;
 
+   // Returns the resources.
+   const XxResources& getResources() const;
+
    // Returns the diffs.
    XxDiffs* getDiffs() const;
 
    // Returns the number of lines in the diffs.
    XxDln getNbLines() const;
 
-   // Returns the font.
-   const QFont& getFont() const;
-
-   // Adjust scrollbars after a resize.
-   void adjustScrollbars( bool force = false );
-
-   // Returns the horizontal position.
-   uint getHorizontalPos() const;
-
-   // Requests for a certain top line to be displayed.  The actual top line that
-   // will be displayed might not be the one that is requested, due to the fact
-   // that we want to avoid letting the user scroll past the end of the file, by
-   // resizing or otherwise).  The actual top line that was computed is
-   // returned.
-   // Out-of-bounds values are permitted.
-   XxDln setTopLine( const XxDln lineNo );
-
-   // Like vertical pos, but tries to center the line instead.  The actual top
-   // line that was computed is returned.
-   XxDln setCenterLine( const XxDln lineNo );
-
-   // Like vertical pos, but tries to set the bottom line instead.  The actual
-   // bottom line that was computed is returned.
-   XxDln setBottomLine( const XxDln lineNo );
-
-   // Sets the cursor line.  Returns the old cursor line.  Value will remain
+   // Get/set the cursor line.  Returns the old cursor line.  Value will remain
    // unchanged if an out-of-bounds value is asked for.  Note: may scroll to
    // insure that the current line is visible.
-   XxDln setCursorLine( const XxDln lineNo );
-
-   // Returns the top/center/bottom/current display line.
    // <group>
-   XxDln getTopLine() const;
-   XxDln getCenterLine() const;
-   XxDln getBottomLine() const;
+   XxDln setCursorLine( const XxDln lineNo, const bool recenter = false );
    XxDln getCursorLine() const;
    // </group>
 
-   // Returns the current number of display lines.
-   XxDln getNbDisplayLines() const;
+   // Make sure that the cursor line is visible between the given boundaries.
+   // Return true if the cursor has been moved.
+   bool adjustCursor( const int topLine, const int bottomLine );
 
-   // Process a main window resize event.
-   void resizeEvent();
+   // Adjusts line number size.
+   void adjustLineNumbers();
 
    // Returns the main window's size.
    QRect getMainWindowGeometry() const;
@@ -197,27 +171,17 @@ public:
    // computeTextWidth().
    uint getTextWidth() const;
 
+   // Forces recomputation of the text width.
+   void invalidateTextWidth();
 
 public slots:
 
    // Redraws the text widgets.
-   void repaintTexts();
+   void updateWidgets();
 
    // On a number of line change in diffs.
    void onNbLinesChanged();
 
-   // Updates the line number labels.
-   void updateLineNumberLabels( int );
-
-   // Scrollbars.
-   // <group>
-   void horizontalScroll( int );
-   void scrollRight();
-   void scrollLeft();
-   void verticalScroll( int );
-   void verticalScroll2( int );
-   // </group>
-
    // This gets called in the main thread when the edit is done.
    void editDone();
 
@@ -229,6 +193,7 @@ public slots:
    void saveAsLeft();
    void saveAsMiddle();
    void saveAsRight();
+   void saveAsMerged();
    void saveAs();
    void saveSelectedOnly();
    void editLeft();
@@ -238,12 +203,6 @@ public slots:
    void search();
    void searchForward();
    void searchBackward();
-   void scrollDown();
-   void scrollUp();
-   void cursorDown();
-   void cursorUp();
-   void cursorTop();
-   void cursorBottom();
    void redoDiff();
    void editDiffOptions();
    void editDisplayOptions();
@@ -265,7 +224,7 @@ public slots:
    void selectGlobalUnselectedMiddle();
    void selectGlobalUnselectedRight();
    void selectGlobalUnselectedNeither();
-   void selectGlobalMerge();
+   int  selectGlobalMerge();
    void selectRegionLeft();
    void selectRegionMiddle();
    void selectRegionRight();
@@ -294,15 +253,17 @@ public slots:
    void qualityNormal();
    void qualityFastest();
    void qualityHighest();
-   void mergedView();
+   void toggleShowFilenames();
+   void togglePaneMergedView();
+   void togglePopupMergedView();
    void toggleToolbar();
    void toggleLineNumbers();
-   void adjustLineNumbers();
-   void toggleShowMarkers();
    void toggleVerticalLine();
    void toggleOverview();
-   void toggleShowFilenames();
-   void toggleHorizontalDiffs();
+   void hordiffTypeNone();
+   void hordiffTypeSingle();
+   void hordiffTypeMultiple();
+   void hordiffTypeUpdate();
    void toggleIgnoreHorizontalWs();
    void toggleFormatClipboardText();
    void ignoreFileNone();
@@ -310,57 +271,48 @@ public slots:
    void ignoreFileMiddle();
    void ignoreFileRight();
    void helpManPage();
-   void helpColorLegend();
    void helpGenInitFile();
    void helpAbout();
    // </group>
-   
+
+   // Keyboard accelerators cursor motion callbacks.
+   // <group>
+   void pageDown(); 
+   void pageUp(); 
+   void cursorDown(); 
+   void cursorUp(); 
+   void cursorTop(); 
+   void cursorBottom(); 
+   // </group>
+
 signals:
 
-   void postCreationAction();
+   // Signal emitted when the cursor changes line.
    void cursorChanged( int cursorLine );
-   void scrolled( int topLine );
+
+   // Signal emitted when the size of the displayed text changes.
+   void textSizeChanged();
 
 private:
 
    /*----- friends -----*/
 
-   friend class XxDiffArgumentsDialog;
-   friend class XxOptionsDialog;
+   friend class XxOptionsDialog; // For non-const access to the resources.
 
    /*----- member functions -----*/
 
-   // Reads the file arguments, returns the number of files to be read, creates
-   // temporary files if necessary (from stdin).
-   uint readFileNames( 
-      int 		argc, 
-      char** 		argv,
-      std::string 	filenames[3],
-      std::string 	displayFilenames[3],
-      bool		created[3],
-      bool&		directories
-   ) const;
-
-   // Parse Xrm and/or rc file and removes the recognized ones.
-   void buildResources();
-
-   // Parse command line options and removes the recognized ones.
-   void parseCommandLine( int& argc, char**& argv );
-
-   // Parse resources specified on command line.
-   void parseCommandLineResources();
+   // Creates a new resources object and parses the rcfile and cmdline
+   // resources.
+   XxResources* buildResources() const;
 
    // Reads in a file.
-   void readFile( 
-      const XxFno no, 
-      const char* filename,
-      const char* displayFilename,
-      bool        isTemporary = false
+   std::auto_ptr<XxBuffer> readFile( 
+      const XxFno    no, 
+      const QString& filename,
+      const QString& displayFilename,
+      bool           isTemporary = false
    );
 
-   // Re-reads a file.
-   void reReadFile( const XxFno no );
-
    // Processes the diff command, be really careful with this.  Returns false if
    // there was an error.
    bool processDiff();
@@ -370,29 +322,25 @@ private:
 
    // Creates the interface.
    // <group>
-   void createUI( uint nbFiles );
+   void createUI();
    void createOnContextHelp();
    void createMenus();
    QToolBar* createToolbar();
    // </group>
 
    // Output diff errors.
-   void outputDiffErrors( const char* errors );
+   void outputDiffErrors( const QString& errors );
 
    // Compute the number of pixels of the longest line.
    uint computeTextWidth() const;
    
-   // Moves the cursor line so that it is visible after a scroll or resize.
-   // Return true if the cursor has been moved.
-   bool adjustCursor();
-
    // Tries to save the selected contents to the specified filename.  If ask is
    // true, ask the user for the filename (and use filename as default value).
-   void saveToFile( const char* filename, const bool ask );
+   void saveToFile( const QString& filename, const bool ask );
 
    // Edits the said file, and add signal handler to catch children exit, and
    // notify.
-   void editFile( const char* filename );
+   void editFile( const QString& filename );
 
    // Reopen file at no and rebuild diffs and update and all.
    void openFile( const XxFno no );
@@ -404,7 +352,7 @@ private:
    void removeFile( XxFno nnno ) const;
 
    // Private non-const version of resource access (for dialogs).
-   XxResources* getResourcesNC();
+   XxResources& getResourcesNC();
 
    // Implements apply-and-move-to-next algorithm.
    void selectAndNext( XxLine::Selection selection );
@@ -414,77 +362,81 @@ private:
    bool validateNeedToSave( uint no ) const;
 
    // Implementation of the ignoreXXX() methods.
-   void setFileDiffOptions( XxResources::Resource );
+   void setFileDiffOptions( XxCommandSwitch );
 
    // Implementation of the qualityXXX() methods.
-   void setQuality( XxResources::Quality );
+   void setQuality( XxQuality );
+
+   // Get the merged filename.
+   QString getMergedFilename() const;
 
    /*----- static member functions -----*/
 
+   // Reads the file arguments, returns the number of files to be read, creates
+   // temporary files if necessary (from stdin).
+   static uint processFileNames( 
+      const XxCmdline& cmdline,
+      QString          filenames[3],
+      QString          displayFilenames[3],
+      bool             created[3],
+      bool&            directories
+   );
+
    // SIGCHLD handler for editor.
    static void handlerSIGCHLD( int );
 
-
-   /*----- data members -----*/
-
-   // UI widgets.
-   bool                    _isUICreated;
-   QStyle*                 _style;
-   QMainWindow*            _mainWindow;
-   QMessageBox*		   _diffErrorsMsgBox;
-   QWidget*                _centralWidget;
-   QPopupMenu*             _viewPopup;
-   QPopupMenu*             _optionsMenu;
-   QPopupMenu*             _displayMenu;
-   QPopupMenu*             _windowsMenu;
-   int 			   _menuids[ MAX_MENUIDS ];
-   XxOverview*             _overview;
-   XxSearchDialog*         _searchDialog;
-   XxOptionsDialog*        _optionsDialog;
-   QLabel*                 _filenameLabel[3];
-   QLabel*                 _lineNumberLabel[3];
-   XxLineNumbers*          _lineNumbers[3];
-   XxText*                 _text[3];
-   XxMergedWindow*         _mergedWindow;
-   QToolBar*               _toolbar;
-   QScrollBar*             _vscroll[2];
-   QScrollBar*             _hscroll;
-   QFont                   _font;
-   uint                    _textWidth;
-   uint                    _displayWidth;  // Last value cached. 
-   uint                    _displayHeight; // Last value cached.
-   XxDln                   _displayLines;
-   XxDln                   _cursorLine;
-                                	
-   int                     _nbFiles;
-   int                     _nbTextWidgets;
-   std::auto_ptr<XxBuffer> _files[3];
-   std::auto_ptr<XxDiffs>  _diffs;
-   bool                    _filesAreDirectories;
-                                	
-   // Resources.                	
-   XxResources*            _resources;
-
-   // Cmdline-related variables.
-   int                     _returnValue;
-   std::string             _userFilenames[3];
-   std::string             _stdinFilename;
-   XxStringResParser*      _stringResParser;
-   bool                    _useXrm;
-   bool                    _useRcfile;
-   std::string             _extraDiffArgs;
-   bool                    _sepDiff;
-
    /*----- static data members -----*/
 
    static int              _sockfd;
    static QSocketNotifier* _socketNotifier;
 
-};
+   /*----- data members -----*/
 
+   // UI widgets.
+   bool                    _isUICreated;
+   bool                    _dontShow;
+   int                     _returnValue;
+
+   QStyle*                 _style;
+   QMainWindow*            _mainWindow;
+   QMessageBox*            _diffErrorsMsgBox;
+   QSplitter*              _splitter;
+   XxMergedFrame*          _paneMergedView;
+   XxMergedWindow*         _popupMergedView;
+   XxCentralFrame*         _central;
+   QPopupMenu*             _viewPopup;
+   QPopupMenu*             _optionsMenu;
+   QPopupMenu*             _displayMenu;
+   QPopupMenu*             _hordiffMenu;
+   QPopupMenu*             _windowsMenu;
+   int                     _menuids[ MAX_MENUIDS ];
+   QWidget*                _overviewArea;
+   QLabel*                 _remUnselView;
+   XxOverview*             _overview;
+   XxSearchDialog*         _searchDialog;
+   XxOptionsDialog*        _optionsDialog;
+   QToolBar*               _toolbar;
+   mutable uint            _textWidth;
+   XxDln                   _cursorLine;
+                                	
+   //
+   // Real data.
+   //
+   int                     _nbFiles;
+   std::auto_ptr<XxBuffer> _files[3];
+   std::auto_ptr<XxDiffs>  _diffs;
+   bool                    _filesAreDirectories;
+
+   XxResources*            _resources;
+
+   // Data from parsing the cmdline.
+   const XxCmdline&        _cmdline;
+
+};
 
 XX_NAMESPACE_END
 
 #include <app.inline.h>
 
 #endif
+
