@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: buffer.inline.h,v 1.11 2002/02/23 21:55:59 blais Exp $
- * $Date: 2002/02/23 21:55:59 $
+ * $Id: buffer.inline.h,v 1.13 2002/02/25 17:08:09 blais Exp $
+ * $Date: 2002/02/25 17:08:09 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -34,8 +34,18 @@ XX_NAMESPACE_BEGIN
 
 //------------------------------------------------------------------------------
 //
-inline const QString& XxBuffer::getName() const
+inline bool XxBuffer::isTemporary() const
 {
+   return _temporary;
+}
+
+//------------------------------------------------------------------------------
+//
+inline QString XxBuffer::getName() const
+{
+   if ( isTemporary() ) {
+      return QString("-");
+   }
    return _name;
 }
 
@@ -46,13 +56,6 @@ inline const QString& XxBuffer::getDisplayName() const
    return _displayName;
 }
 
-//------------------------------------------------------------------------------
-//
-inline bool XxBuffer::isTemporary() const
-{
-   return _temporary;
-}
-
 //------------------------------------------------------------------------------
 //
 inline uint XxBuffer::getNbLines() const
