@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: builderDirs2.h,v 1.6 2002/02/23 17:43:56 blais Exp $
- * $Date: 2002/02/23 17:43:56 $
+ * $Id: builderDirs2.h,v 1.7 2002/02/25 00:17:30 blais Exp $
+ * $Date: 2002/02/25 00:17:30 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -84,10 +84,8 @@ public:
    // may want to check hasErrors() if you want to display those.
    std::auto_ptr<XxDiffs> process(
       const QString& command,
-      const QString& path1,
-      XxBuffer*      buffer1,
-      const QString& path2,
-      XxBuffer*      buffer2
+      XxBuffer&      buffer1,
+      XxBuffer&      buffer2
    );
 
 private:
