@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: builderFiles2.h,v 1.8 2002/02/23 17:43:56 blais Exp $
- * $Date: 2002/02/23 17:43:56 $
+ * $Id: builderFiles2.h,v 1.9 2002/02/25 00:17:30 blais Exp $
+ * $Date: 2002/02/25 00:17:30 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -51,6 +51,12 @@
 
 XX_NAMESPACE_BEGIN
 
+/*==============================================================================
+ * FORWARD DECLARATIONS
+ *============================================================================*/
+
+class XxBuffer;
+
 /*==============================================================================
  * CLASS XxBuilderFiles2
  *============================================================================*/
@@ -79,11 +85,9 @@ public:
    // allocated XxDiffs object.  However, there might be warnings, so you may
    // want to check hasErrors() if you want to display those.
    std::auto_ptr<XxDiffs> process(
-      const QString& command,
-      const QString& path1,
-      const uint     nbLines1,
-      const QString& path2,
-      const uint     nbLines2
+      const QString&  command,
+      const XxBuffer& buffer1,
+      const XxBuffer& buffer2
    );
 
 private:
