@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: builderFiles3.h,v 1.7 2002/02/23 17:43:56 blais Exp $
- * $Date: 2002/02/23 17:43:56 $
+ * $Id: builderFiles3.h,v 1.8 2002/02/25 00:17:30 blais Exp $
+ * $Date: 2002/02/25 00:17:30 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -51,6 +51,12 @@
 
 XX_NAMESPACE_BEGIN
 
+/*==============================================================================
+ * FORWARD DECLARATIONS
+ *============================================================================*/
+
+class XxBuffer;
+
 /*==============================================================================
  * CLASS XxBuilderFiles3
  *============================================================================*/
@@ -77,12 +83,9 @@ public:
    // want to check hasErrors() if you want to display those.
    std::auto_ptr<XxDiffs> process(
       const QString& command,
-      const QString& path1,
-      const uint     nbLines1,
-      const QString& path2,
-      const uint     nbLines2,
-      const QString& path3,
-      const uint     nbLines3
+      const XxBuffer& buffer1,
+      const XxBuffer& buffer2,
+      const XxBuffer& buffer3
    );
 
 private:
