@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: cmdline.cpp,v 1.33 2002/02/12 02:32:31 blais Exp $
- * $Date: 2002/02/12 02:32:31 $
+ * $Id: cmdline.cpp,v 1.35 2002/02/25 06:57:14 blais Exp $
+ * $Date: 2002/02/25 06:57:14 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -152,6 +152,11 @@ XxCmdline::Option XxCmdline::_optionsXxdiff[] = {
      "conflicts into two files for display.  If this is specified, only a "
      "single file can then be given as argument."
    }, 
+   { "unmerge3", 0, false, 'u', 
+     "Invoke on a single file with merge conflicts generated by diff3 with the "
+     "show-all option, splitting the conflicts into three files for display. "
+     "If this is specified, only a single file can then be given as argument."
+   }, 
    { "title1", 0, true, '1', 
      "Display 'str' instead of filename in filename label 1 (left)."
    }, 
@@ -288,6 +293,7 @@ XxCmdline::XxCmdline() :
    _useRcfile( true ), 
    _extraDiffArgs( "" ),
    _unmerge( false ),
+   _unmergeNbFiles( 2 ),
    _nbQtOptions( 0 ),
    _qtOptions()
 {
@@ -435,6 +441,12 @@ bool XxCmdline::parseCommandLine( const int argc, char* const* argv )
 
          case 'U': {
             _unmerge = true;
+            _unmergeNbFiles = 2;
+         } break;
+
+         case 'u': {
+            _unmerge = true;
+            _unmergeNbFiles = 3;
          } break;
 
          case '1':
@@ -630,15 +642,6 @@ bool XxCmdline::parseCommandLine( const int argc, char* const* argv )
       throw XxUsageError( XX_EXC_PARAMS, msg );
    }
 
-#if 0 
-   // FIXME remove
-
-   // Disable conflict until implemented.
-   if ( _unmerge ) {
-      throw XxUsageError( XX_EXC_PARAMS, "Not yet implemented." );
-   }
-#endif
-
    // Read filenames.
    int ii;
    for ( ii = 0; ii < _nbFilenames; ++ii ) {
