@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: line.h,v 1.5 2001/06/04 05:08:52 blais Exp $
- * $Date: 2001/06/04 05:08:52 $
+ * $Id: line.h,v 1.21 2002/02/20 17:09:20 blais Exp $
+ * $Date: 2002/02/20 17:09:20 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *****************************************************************************/
+ ******************************************************************************/
 
 #ifndef INCL_XXDIFF_LINE
 #define INCL_XXDIFF_LINE
@@ -35,10 +35,23 @@
 #include <types.h>
 #endif
 
+#ifndef INCL_XXDIFF_RESOURCES
+#include <resources.h>
+#define INCL_XXDIFF_RESOURCES
+#endif
+
+#ifndef INCL_QT_QSTRING
+#include <qstring.h>
+#define INCL_QT_QSTRING
+#endif
+
+#ifndef INCL_STD_IOSTREAM
+#include <iostream>
+#define INCL_STD_IOSTREAM
+#endif
 
 XX_NAMESPACE_BEGIN
 
-
 /*==============================================================================
  * CLASS XxLine
  *============================================================================*/
@@ -107,6 +120,12 @@ public:
 
    /*----- member functions -----*/
 
+   // Default constructor, initialize to an invalid line.
+   XxLine();
+
+   // Copy constructor.
+   XxLine( const XxLine& copy );
+
    // Constructor (3-way).
    // For DIFF regions who have one side blank just specify -1 for that side.
    XxLine( 
@@ -116,11 +135,19 @@ public:
       XxFln fline2 = -1
    );
 
+   // Destructor, non-virtual on purpose (gotta keep things small, this goes
+   // into a vector).
+   ~XxLine();
+
    // Returns the line number in the file NOT in the diffs.  If the line for
    // the requested file nb. does not correspond to any actual file text, return
    // -1.
    XxFln getLineNo( const XxFno no ) const;
 
+   // Allows one to change a line no. Do not use this, this is meant to be used
+   // by the reindexing code only, for the unmerged support.
+   void setLineNo( const XxFno no, const XxFln fline );
+
    // Get the type.
    Type getType() const;
 
@@ -137,22 +164,19 @@ public:
 
    // Compute (initialize) horizontal brackets.
    void initializeHorizontalDiff( 
-      const bool  ignoreWs,
-      const char* text0,
-      const uint  len0,
-      const char* text1,
-      const uint  len1,
-      const char* text2 = 0,
-      const uint  len2 = 0
+      const XxResources& resources,
+      const char*        text[3],
+      const uint         len[3]
    );
 
    // Returns true if this line has horizontal diffs.
    bool hasHorizontalDiffs( const XxFno no ) const;
 
-   // Returns the left/right horizontal diff position (in chars) for side no.
+   // Returns the left/right horizontal diff list for a particular file.  Format
+   // is a list of ints, and could be possibly null, ends with -1 element if not
+   // null, and there is always an even number of valid brackets.
    // <group>
-   int getLeftHdiffPos( const XxFno no ) const;
-   int getRightHdiffPos( const XxFno no ) const;
+   const int* getHorDiffs( const XxFno no ) const;
    // </group>
 
    // Get/set the hunk id.
@@ -169,6 +193,15 @@ public:
    // used during merging of lines.
    XxLine getPromoted( Type promoteType ) const;
 
+   // Get display color for line text.  Returns the resource for the background
+   // color.
+   void getLineColorType(
+      const XxIgnoreFile ignoreFile,
+      const XxFno        no,
+      XxColor&           dtype,
+      XxColor&           dtypeSup
+   ) const;
+
    /*----- static member functions -----*/
 
    // Returns true if the two types are part of the same region.
@@ -189,40 +222,48 @@ public:
 
    // Debugging output functions.
    // <group>
-   static std::string mapToString( Type type );
-   static std::string mapToString( Selection selection );
+   static QString mapToString( Type type );
+   static QString mapToString( Selection selection );
    // </group>
 
    // Maps a line type to a file no, when possible. Returns -1 if none makes
    // sense.
    static XxFno mapTypeToFileNo( Type type );
 
-   /*----- static data members -----*/
-
-   static Type _ignoreConvertTables[ 4 ][ NB_TYPES ];
-   
 private:
 
    /*----- member functions -----*/
 
    void init();
 
+   // Implementations of the line color type algorithms.
+   // <group>
+   bool getLineColorIfSelected(
+      const XxFno no,
+      XxColor&    dtype,
+      XxColor&    dtypeSup
+   ) const;
+   void getLineColorTypeStd(
+      const XxLine::Type newType,
+      const XxFno        no,
+      XxColor&           dtype,
+      XxColor&           dtypeSup
+   ) const;
+   // </group>
+
+   /*----- static data members -----*/
+
+   static Type _ignoreConvertTables[ 4 ][ NB_TYPES ];
+
    /*----- data members -----*/
 
-   XxFln	_lineNo[3];
-   Type		_type;
-   Selection	_selection;
-   XxHunk	_hunkId; // This is used for split/swap/join.
+   XxFln       _lineNo[3];
+   Type        _type;
+   Selection   _selection;
+   XxHunk      _hunkId; // This is used for split/swap/join.
 
-   // horizontal diff brackets
-
-   // This is the first different character (may be length)
-   int		_lefthd[3];
-   // This is the last different character (may be -1)
-   int		_righthd[3];
-
-   // This means that a line without horizontal differences will have
-   // (righthd-1) == lefthd.
+   // Horizontal diff brackets.
+   const int*  _hordiffs[3];
 
 };
 
