@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: lineNumbers.cpp,v 1.4 2001/06/04 01:23:53 blais Exp $
- * $Date: 2001/06/04 01:23:53 $
+ * $Id: lineNumbers.cpp,v 1.17 2002/02/20 17:09:20 blais Exp $
+ * $Date: 2002/02/20 17:09:20 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *****************************************************************************/
+ ******************************************************************************/
 
 /*==============================================================================
  * EXTERNAL DECLARATIONS
@@ -26,6 +26,7 @@
 
 #include <lineNumbers.h>
 #include <app.h>
+#include <central.h>
 #include <resources.h>
 #include <diffs.h>
 #include <buffer.h>
@@ -55,19 +56,25 @@ XX_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
 //
 XxLineNumbers::XxLineNumbers( 
-   XxApp*      app, 
-   const XxFno no, 
-   QWidget*    parent, 
-   const char* name 
+   XxApp*          app, 
+   XxCentralFrame* central, 
+   const XxFno     no, 
+   QWidget*        parent, 
+   const char*     name 
 ) :
    QFrame( parent, name, WResizeNoErase ),
    _app( app ),
+   _central( central ),
    _no( no )
 {
    // This must be set equal to the one in XxText for proper vertical alignment
    // of text.
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
    setLineWidth( 2 );
+
+   setSizePolicy(
+      QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding )
+   );
 }
 
 //------------------------------------------------------------------------------
@@ -76,13 +83,6 @@ XxLineNumbers::~XxLineNumbers()
 {
 }
 
-//------------------------------------------------------------------------------
-//
-QSizePolicy XxLineNumbers::sizePolicy() const
-{
-   return QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Expanding );
-}
-
 //------------------------------------------------------------------------------
 //
 void XxLineNumbers::drawContents( QPainter* pp )
@@ -98,28 +98,34 @@ void XxLineNumbers::drawContents( QPainter* pp )
    p.setViewport( rect );
    rect.moveBy( -rect.x(), -rect.y() );
    p.setWindow( rect );
+//#define BACK_COLORING
+#ifdef BACK_COLORING
+   const int w = rect.width();
+#endif
+   const XxResources& resources = _app->getResources();
 
-   XxBuffer* file = _app->getFile( _no );
+   XxBuffer* file = _app->getBuffer( _no );
    const XxDiffs* diffs = _app->getDiffs();
-   const XxResources* resources = XxResources::getInstance();
+   if ( diffs == 0 ) {
+      return;
+   }
 
    //
    // Draw appropriate content.
    //
-   uint topLine = _app->getTopLine();
+   uint topLine = _central->getTopLine();
 
    // Font.
-   p.setFont( _app->getFont() );
+   p.setFont( resources.getFontText() );
    QFontMetrics fm = p.fontMetrics();
 
    QPen pen;
 
-   uint displayLines = _app->getNbDisplayLines();
-   uint nbLines = 
-      std::min( displayLines, diffs->getNbLines() - (topLine - 1) );
+   uint displayLines = _central->getNbDisplayLines();
+   uint nbLines = std::min( displayLines, diffs->getNbLines() - (topLine - 1) );
 
-   char lnFormat[16];
-   ::snprintf( lnFormat, sizeof(lnFormat), "%%%dd", _app->getMaxDigits() );
+   QString lnFormat;
+   lnFormat.sprintf( "%%%dd", _app->getMaxDigits() );
 
    const int x = 0;
    int y = 0;
@@ -128,22 +134,32 @@ void XxLineNumbers::drawContents( QPainter* pp )
       // Get line to display.
       const XxLine& line = diffs->getLine( topLine + ii );
 
+      // This is ugly.  I prefer line numbers without back coloring, it makes it
+      // less confusing for actual file text. Keep for reference only.
+#ifdef BACK_COLORING
       // Set background and foreground colors.
       QColor bcolor;
       QColor fcolor;
-      XxResources::Resource dtype, dtypeSup;
-      resources->getLineColorType( line, _no, dtype, dtypeSup );
-      resources->getRegionColor( dtype, bcolor, fcolor );
+      XxColor dtype, dtypeSup;
+      line.getLineColorType( 
+         resources.getIgnoreFile(),
+         _no,
+         dtype, dtypeSup
+      );
+      resources.getRegionColor( dtype, bcolor, fcolor );
       QBrush brush( bcolor );
+      p.fillRect( x, y, w, fm.lineSpacing(), brush );
+#endif
 
       // Render text.
-      int fline = line.getLineNo( _no );
+      XxFln fline = line.getLineNo( _no );
       if ( fline != -1 ) {
+         XxFln dfline = file->getDisplayLineNo( fline );
 
-         const char* renderedNums = file->renderLineNumber( fline, lnFormat );
-         QString str( renderedNums );
-         
-         p.drawText( x, y + fm.ascent(), str );
+         const QString& renderedNums =
+            file->renderLineNumber( dfline, lnFormat );
+
+         p.drawText( x, y + fm.ascent(), renderedNums );
       }
    }
 
