@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: optionsDialog.h,v 1.9 2001/06/04 20:16:12 blais Exp $
- * $Date: 2001/06/04 20:16:12 $
+ * $Id: optionsDialog.h,v 1.18 2001/11/30 07:21:57 blais Exp $
+ * $Date: 2001/11/30 07:21:57 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *****************************************************************************/
+ ******************************************************************************/
 
 #ifndef INCL_XXDIFF_OPTIONSDIALOG
 #define INCL_XXDIFF_OPTIONSDIALOG
@@ -69,12 +69,12 @@ public:
    // Constructor.
    XxOptionsDialog( XxApp* app, QWidget* parent );
 
+   // Destructor.
+   virtual ~XxOptionsDialog();
+
    // See base class.
    virtual void show();
 
-   // Asks the dialog to synchronize itself.
-   void synchronize();
-
    // Selects a particular tab. 
    // <group>
    void selectDiffOptions();
@@ -86,17 +86,17 @@ public:
    // Methods to manipulate command strings.
    // <group>
    static bool isInCommand(
-      const std::string& command,
-      const char*        option
+      const QString& command,
+      const QString& option
    );
 
    static void addToCommand(
-      std::string& command,
-      const char*  option
+      QString&       command,
+      const QString& option
    );
    static void removeFromCommand( 
-      std::string& command,
-      const char*  option
+      QString&       command,
+      const QString& option
    );
 
    // Look for one of three options. If found, return 1, 2 or 3. If neither is
@@ -104,18 +104,18 @@ public:
    // none of the non-empty options is found, return the index of the empty
    // string option.
    static int isInCommand(
-      const std::string& command,
-      const char*        option1,
-      const char*        option2,
-      const char*        option3
+      const QString& command,
+      const QString& option1,
+      const QString& option2,
+      const QString& option3
    );
 
    // Set one option among three. One of the options may be the empty string.
    static void setOneOfInCommand(
-      std::string& command,
-      const char*  optionNew,
-      const char*  optionOther1,
-      const char*  optionOther2
+      QString&       command,
+      const QString& optionNew,
+      const QString& optionOther1,
+      const QString& optionOther2
    );
    // </group>
 
@@ -123,6 +123,9 @@ public slots:
 
    /*----- member functions -----*/
 
+   // Asks the dialog to synchronize itself.
+   void synchronize();
+
    void onApply();
    void onClose();
 
@@ -143,59 +146,62 @@ public slots:
    //---------------------------------------------------------------------------
    // Directories
 
-   void checkboxRecursive( int );
+   //     void checkboxRecursive( int );
 
    //---------------------------------------------------------------------------
    // Display
 
-   void checkboxHorizontalDiffs( int );
-   void checkboxIgnoreHorizontalWhitespace( int );
-   void checkboxHideCarriageReturns( int );
-   void spinboxTabWidth( int );
-   void checkboxIgnoreFileChanges( int );
-   void checkboxFormatClipboardText( int );
-   void checkboxDrawVerticalLine( int );
-   void comboIgnoreFile( const QString& );
+   //     void checkboxHorizontalDiffs( int );
+   //     void checkboxIgnoreHorizontalWhitespace( int );
+   //     void checkboxHideCarriageReturns( int );
+   //     void spinboxTabWidth( int );
+   //     void checkboxIgnoreFileChanges( int );
+   //     void checkboxFormatClipboardText( int );
+   //     void checkboxDrawVerticalLine( int );
+   //     void comboIgnoreFile( const QString& );
+
+   void editFontApp();
+   void editFontText();
 
    //---------------------------------------------------------------------------
    // Colors
 
    void listboxColors( const QString& );
-   void editColor();
+   void editColorFore();
+   void editColorBack();
 
 
 private:
 
-   /*----- types and enumerationse -----*/
-
-   typedef std::map< int, QColor > ColorModMap;
-
    /*----- member functions -----*/
 
    int getDiffPageIndex() const;
 
    void setFileDiffOptions(
-      XxResources::Resource cmdOptionId,
-      bool                  state
+      XxCommandSwitch cmdOptionId,
+      bool         state
    ) const;
 
    void setFileDiffQuality(
-      XxResources::Quality quality
+      XxQuality quality
    ) const;
 
    bool maybeSetCommand( 
-      XxResources::Resource commandId,
-      const char*           commandString
+      XxCommand      commandId,
+      const QString& commandString
    );
 
-
    /*----- data members -----*/
 
-   XxApp*      _app;
-   ColorModMap _colorModMap;
+   XxApp* _app;
+
+   // Note: perhaps we should keep a whole resources object for the dialog.
+   QFont  _fontApp;
+   QFont  _fontText;
 
 };
 
 XX_NAMESPACE_END
 
 #endif
+
