@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: resParser.cpp,v 1.54 2002/02/19 19:13:57 blais Exp $
- * $Date: 2002/02/19 19:13:57 $
+ * $Id: resParser.cpp,v 1.56 2002/02/27 02:46:54 blais Exp $
+ * $Date: 2002/02/27 02:46:54 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -509,6 +509,11 @@ the start of a merged merge conflict. Default is the CVS start tag." },
      "Tag that appears at the beginning of a line in a merged file to separate \
 sides of a merged merge conflict. Default is the CVS start tag." },
 
+   { "Unmerge.SepExtra", TAG_UNMERGE_SEP_EXTRA, 
+     "Tag that appears at the beginning of a line in a merged file to separate \
+sides of a merged 3-way merge conflict. Default is the tag generated by GNU \
+diff3." },
+
    { "Unmerge.End", TAG_UNMERGE_END, 
      "Tag that appears at the beginning of a line in a merged file to indicate \
 the end of a merged merge conflict. Default is the CVS start tag." },
@@ -755,6 +760,9 @@ char lexerBuffer[ LEX_BUFFER_MAX ];
 #pragma reset woff 1506
 #endif
 
+// Prevent warning under IRIX and Linux.
+char* dummy_full_match = yy_full_match;
+
 //------------------------------------------------------------------------------
 //
 void yyerror( const char* msg )
