@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: resources.inline.h,v 1.9 2001/06/02 01:26:38 blais Exp $
- * $Date: 2001/06/02 01:26:38 $
+ * $Id: resources.inline.h,v 1.28 2002/01/31 02:42:05 blais Exp $
+ * $Date: 2002/01/31 02:42:05 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -18,16 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *****************************************************************************/
+ ******************************************************************************/
 
 /*==============================================================================
  * EXTERNAL DECLARATIONS
  *============================================================================*/
 
-
 XX_NAMESPACE_BEGIN
 
-
 /*==============================================================================
  * CLASS XxResources
  *============================================================================*/
@@ -41,81 +39,112 @@ inline const QRect& XxResources::getPreferredGeometry() const
 
 //------------------------------------------------------------------------------
 //
-inline int XxResources::getAccelerator( Resource resource ) const
+inline bool XxResources::getMaximize() const
 {
-   int iaccel = int( resource ) - int( ACCEL_FIRST );
-   XX_CHECK( iaccel < int(ACCEL_LAST) - int(ACCEL_FIRST) );
-   return _accelerators[ iaccel ];
+   return _maximize;
 }
 
 //------------------------------------------------------------------------------
 //
-inline const QFont& XxResources::getAppFont() const
+inline int XxResources::getAccelerator( XxAccel accel ) const
+{
+   return _accelerators[ int(accel) ];
+}
+
+//------------------------------------------------------------------------------
+//
+inline const QFont& XxResources::getFontApp() const
 {
    return _fontApp;
 }
 
 //------------------------------------------------------------------------------
 //
-inline const QFont& XxResources::getTextFont() const
+inline const QFont& XxResources::getFontText() const
 {
    return _fontText;
 }
 
 //------------------------------------------------------------------------------
 //
-inline void XxResources::getRegionColor(
-   Resource type,
-   QColor&  background,
-   QColor&  foreground
+inline XxIgnoreFile XxResources::getIgnoreFile() const
+{
+   return _ignoreFile;
+}
+
+//------------------------------------------------------------------------------
+//
+inline const QString& XxResources::getCommand( XxCommand cmdId ) const
+{
+   return _commands[ int(cmdId) ];
+}
+
+//------------------------------------------------------------------------------
+//
+inline const QString& XxResources::getCommandSwitch(
+   XxCommandSwitch cmdId
 ) const
 {
-   int i = int(type) - int(COLOR_FIRST);
-   XX_CHECK( i < int(COLOR_LAST) - int(COLOR_FIRST) );
-   XX_CHECK( i < int(COLOR_BACKGROUND) );
-   // XX_CHECK( ( i - int(COLOR_FIRST) ) % 2 == 0 );
-   background = _colors[ i ];
-   foreground = _colors[ i + 1 ];
+   return _commandSwitch[ int(cmdId) ];
 }
 
 //------------------------------------------------------------------------------
 //
-inline const QColor& XxResources::getColor( const Resource resource ) const
+inline int XxResources::getInitSwitch(
+   XxCommandSwitch cmdId
+) const
 {
-   int i = int(resource) - int(COLOR_FIRST);
-   XX_CHECK( i < int(COLOR_LAST) - int(COLOR_FIRST) );
-   return _colors[ i ];
+   return _initSwitch[ int(cmdId) ];
 }
 
 //------------------------------------------------------------------------------
 //
-inline bool XxResources::getBoolOpt( const Resource resource ) const
+inline const QString& XxResources::getTag( XxTag tagId ) const
 {
-   int ibool = int( resource ) - int( BOOL_FIRST );
-   XX_CHECK( ibool < int(BOOL_LAST) - int(BOOL_FIRST) );
-   return _boolOpts[ ibool ];
+   return _tags[ int(tagId) ];
 }
 
 //------------------------------------------------------------------------------
 //
-inline void XxResources::setBoolOpt(
-   const Resource resource,
-   const bool option
-)
+inline void XxResources::getRegionColor(
+   const XxColor colorType,
+   QColor&       background,
+   QColor&       foreground
+) const
 {
-   int ibool = int( resource ) - int( BOOL_FIRST );
-   XX_CHECK( ibool < int(BOOL_LAST) - int(BOOL_FIRST) );
-   _boolOpts[ ibool ] = option;
+   int i = int(colorType);
+   background = _backColors[ i ];
+   foreground = _foreColors[ i ];
 }
 
 //------------------------------------------------------------------------------
 //
-inline bool XxResources::toggleBoolOpt( const Resource resource )
+inline const QColor& XxResources::getColor(
+   const XxColor colorType,
+   const bool    fore
+) const
 {
-   int ibool = int( resource ) - int( BOOL_FIRST );
-   XX_CHECK( ibool < int(BOOL_LAST) - int(BOOL_FIRST) );
-   _boolOpts[ ibool ] = ! _boolOpts[ ibool ];
-   return _boolOpts[ ibool ];
+   if ( fore ) {
+      return _foreColors[ int(colorType) ];
+   }
+   else {
+      return _backColors[ int(colorType) ];
+   }
+}
+
+//------------------------------------------------------------------------------
+//
+inline bool XxResources::getBoolOpt( const XxBoolOpt opt ) const
+{
+   XX_ASSERT( 0 <= opt && opt < BOOL_LAST );
+   return _boolOpts[ int(opt) ];
+}
+
+//------------------------------------------------------------------------------
+//
+inline bool XxResources::getShowOpt( const XxShowOpt opt ) const
+{
+   return _showOpts[ int(opt) ];
 }
 
 //------------------------------------------------------------------------------
@@ -125,20 +154,6 @@ inline uint XxResources::getTabWidth() const
    return _tabWidth;
 }
 
-//------------------------------------------------------------------------------
-//
-inline void XxResources::setTabWidth( uint tabWidth )
-{
-   _tabWidth = tabWidth;
-}
-
-//------------------------------------------------------------------------------
-//
-inline void XxResources::setVerticalLinePos( uint vlinePos )
-{
-   _verticalLinePos = vlinePos;
-}
-
 //------------------------------------------------------------------------------
 //
 inline uint XxResources::getOverviewFileWidth() const
@@ -162,30 +177,99 @@ inline uint XxResources::getVerticalLinePos() const
 
 //------------------------------------------------------------------------------
 //
-inline const std::string& XxResources::getClipboardTextFormat() const
+inline const QString& XxResources::getClipboardFormat() const
 {
-   return _clipboardTextFormat;
+   return _clipboardFormat;
 }
 
 //------------------------------------------------------------------------------
 //
-inline void XxResources::setClipboardTextFormat( const std::string& format )
+inline XxHordiff XxResources::getHordiffType() const
 {
-   _clipboardTextFormat = format;
+   return _hordiffType;
 }
 
 //------------------------------------------------------------------------------
 //
-inline XxResources::IgnoreFile XxResources::getIgnoreFile() const
+inline uint XxResources::getHordiffMax() const
 {
-   return _ignoreFile;
+   return _hordiffMax;
 }
 
 //------------------------------------------------------------------------------
 //
-inline void XxResources::setIgnoreFile( IgnoreFile ignoreFile )
+inline uint XxResources::getHordiffContext() const
 {
-   _ignoreFile = ignoreFile;
+   return _hordiffContext;
+}
+
+//------------------------------------------------------------------------------
+//
+inline uint XxResources::getShowPaneMergedViewPercent() const
+{
+   return _showPaneMergedViewPercent;
+}
+
+//------------------------------------------------------------------------------
+//
+inline const QString& XxResources::getMergedFilename() const
+{
+   return _mergedFilename;
+}
+
+//------------------------------------------------------------------------------
+//
+inline void XxResources::setFbColors(
+   XxColor     color,
+   const char* backstr,
+   const char* forestr
+)
+{
+   _backColors[ color ].setNamedColor( backstr );
+   _foreColors[ color ].setNamedColor( forestr );
+   // Note: no need to emit changed(), this is a convenience method for the
+   // inside.
+}
+
+//------------------------------------------------------------------------------
+//
+inline int* XxResources::getDynProgTable( const uint htx, const uint hty ) const
+{
+   if ( ( htx * hty ) > _hdTableSize ) {
+      // We won't start messing with large diffs, allow a maximum table size
+      // larger than a constant number.
+      if ( (htx * hty) > _hordiffMax ) {
+         return 0;
+      }
+      _hdTableSize = htx * hty;
+      _hdTable = (int*) realloc( _hdTable, _hdTableSize * sizeof(int) );
+   }
+   return _hdTable;
+}
+
+//------------------------------------------------------------------------------
+//
+inline uint XxResources::getHordiffBuffers(
+   int*&  hbuffer0, 
+   int*&  hbuffer1
+) const
+{
+   hbuffer0 = _hdBuffer0;
+   hbuffer1 = _hdBuffer1;
+   return _hdBufferSize;
+}
+
+//------------------------------------------------------------------------------
+//
+inline uint XxResources::getHordiffLineBuffers(
+   char*&  hdlinebuf0, 
+   char*&  hdlinebuf1
+) const
+{
+   hdlinebuf0 = _hdLineBuf0;
+   hdlinebuf1 = _hdLineBuf1;
+   return _hdLineBufSize;
 }
 
 XX_NAMESPACE_END
+
