@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: searchDialog.cpp,v 1.11 2001/11/30 07:21:57 blais Exp $
- * $Date: 2001/11/30 07:21:57 $
+ * $Id: searchDialog.cpp,v 1.12 2002/02/25 06:57:14 blais Exp $
+ * $Date: 2002/02/25 06:57:14 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -63,6 +63,11 @@ XxSearchDialog::XxSearchDialog(
 
    connect( _buttonGotoLine, SIGNAL( clicked() ),
             this, SLOT( onGotoLine() ) );
+
+
+   if ( _app->getNbFiles() == 2 ) {
+      _comboGotoWhichFile->removeItem( 2 );
+   }
 }
 
 //------------------------------------------------------------------------------
@@ -91,8 +96,11 @@ void XxSearchDialog::onGotoLine()
    }
 
    XxDiffs* diffs = _app->getDiffs();
-   XxDln dline = diffs->getDisplayLine( gline, fno );
-   _app->setCursorLine( dline );
+   XxBuffer* buffer = _app->getBuffer( fno );
+   XxDln dline = diffs->getDisplayLine( gline, *buffer, fno );
+   if ( dline != -1 ) {
+      _app->setCursorLine( dline );
+   }
 }
 
 //------------------------------------------------------------------------------
