@@ -1,6 +1,6 @@
 /******************************************************************************\
- * $Id: util.h,v 1.24 2002/02/07 20:10:05 blais Exp $
- * $Date: 2002/02/07 20:10:05 $
+ * $Id: util.h,v 1.25 2002/02/25 00:17:30 blais Exp $
+ * $Date: 2002/02/25 00:17:30 $
  *
  * Copyright (C) 1999-2001  Martin Blais <blais@iro.umontreal.ca>
  *
@@ -99,11 +99,14 @@ public:
    // standard error have been redirected.
    //
    // If you don't want the pipes, let outf and/or errf be null.
+   //
+   // If cstdin is not null, it is piped into the subprocess standard input.
    static void spawnCommand( 
       const char** argv,
       FILE** outf,
       FILE** errf,
-      void (*sigChldHandler)(int) = 0
+      void (*sigChldHandler)(int) = 0,
+      const char* cstdin = 0
    );
 
    // Convenience for above method, without output.
